@@ -3,7 +3,7 @@
 ## Copyright (C) 2002 - 2009 Dirk Eddelbuettel <edd@debian.org>
 ## Copyright (C) 2009        Khanh Nguyen <knguyen@cs.umb.edu>
 ##
-## $Id: bond.R 102 2009-08-06 15:09:19Z knguyen $
+## $Id: bond.R 261 2010-06-19 21:33:44Z edd $
 ##
 ## This file is part of the RQuantLib library for GNU R.
 ## It is made available under the terms of the GNU General Public
@@ -25,31 +25,31 @@ ZeroCouponBond <- function(bond, discountCurve, dateparams ) {
     UseMethod("ZeroCouponBond")
 }
 
-ZeroCouponBond.default <- function(bond, discountCurve, dateparams) {
+ZeroCouponBond.default <- function(bond,
+                                   discountCurve,
+                                   dateparams=list(
+                                     refDate=bond$issueDate,
+                                     settlementDays=1,
+                                     calendar='us',
+                                     businessDayConvention='Following')) {
     val <- 0
+
+    if (is.null(bond$faceAmount)) {bond$faceAmount=100}
+    if (is.null(bond$redemption)) {bond$redemption=100}
+
+    if (is.null(dateparams$settlementDays)) {dateparams$settlementDays=1}
+    if (is.null(dateparams$calendar)) {dateparams$calendar='us'}
+    if (is.null(dateparams$businessDayConvention)) {dateparams$businessDayConvention='Following'}
+    if (is.null(dateparams$refDate)) {dateparams$refDate=bond$issueDate}
     dateparams <- matchParams(dateparams)
-    if (class(discountCurve)=="DiscountCurve"){
-        val <- .Call("QL_ZeroBondWithRebuiltCurve",
-                  bond, c(discountCurve$table$date), 
-                  discountCurve$table$zeroRates, dateparams,
-                  PACKAGE="RQuantLib")
-    }
-    else{
-        if (length(discountCurve)==2){
-            val <- .Call("QL_ZBond1", 
-                     bond, discountCurve, dateparams,
-                     PACKAGE="RQuantLib")
-        }
-        if (length(discountCurve)==3){
-             val <- .Call("QL_ZBond2", 
-                    bond, discountCurve[[1]], 
-                    discountCurve[[2]], discountCurve[[3]],
-                    dateparams,
-                    PACKAGE="RQuantLib")      
-        }
-    }
-    val$cashFlow <- as.data.frame(val$cashFlow)
-    class(val) <- c("ZeroCouponBond", "Bond")    
+
+
+    val <- .Call("QL_ZeroBondWithRebuiltCurve",
+                 bond, c(discountCurve$table$date),
+                 discountCurve$table$zeroRates, dateparams,
+                 PACKAGE="RQuantLib")
+
+    class(val) <- c("ZeroCouponBond", "Bond")
     val
 }
 
@@ -62,7 +62,7 @@ ZeroPriceByYield <- function(yield, faceAmount,
           UseMethod("ZeroPriceByYield")
 }
 
-ZeroPriceByYield.default <- function(yield, faceAmount,
+ZeroPriceByYield.default <- function(yield, faceAmount=100,
                               issueDate, maturityDate,
                               dayCounter=2, frequency=2,
                               compound=0, businessDayConvention=4){
@@ -88,7 +88,7 @@ ZeroYield <- function(price, faceAmount,
                       compound,	businessDayConvention){
 	  UseMethod("ZeroYield")
 }
-ZeroYield.default <- function(price, faceAmount,
+ZeroYield.default <- function(price, faceAmount=100,
                               issueDate, maturityDate,
                               dayCounter=2, frequency=2,
                               compound=0, businessDayConvention=4){
@@ -113,31 +113,50 @@ ZeroYield.default <- function(price, faceAmount,
 FixedRateBond <- function(bond, rates, discountCurve, dateparams){
      UseMethod("FixedRateBond")
 }
-FixedRateBond.default <- function(bond, rates, discountCurve, dateparams){
+FixedRateBond.default <- function(bond,
+                                  rates,
+                                  discountCurve,
+                                  dateparams=list(
+
+                                    settlementDays=1,
+                                    calendar='us',
+                                    businessDayConvention='Following',
+                                    terminationDateConvention='Following',
+                                    dayCounter='Thirty360',
+                                    period='Semiannual',
+                                    dateGeneration='Backward',
+                                    endOfMonth=0,
+                                    fixingDays=2
+                                    )){
     val <- 0
+
+    if (is.null(bond$faceAmount)){bond$faceAmount=100}
+    if (is.null(bond$redemption)){bond$redemption=100}
+    if (is.null(bond$effectiveDate)){bond$effectiveDate=bond$issueDate}
+
+
+    if (is.null(dateparams$settlementDays)){dateparams$settlementDays=1}
+    if (is.null(dateparams$calendar)){dateparams$calendar='us'}
+    if (is.null(dateparams$businessDayConvention)){
+      dateparams$businessDayConvention='Following'
+    }
+    if (is.null(dateparams$terminationDateConvention)){
+      dateparams$terminationDateConvention='Following'
+    }
+    if (is.null(dateparams$dayCounter)){dateparams$dayCounter='Thirty360'}
+    if (is.null(dateparams$period)){dateparams$period='Semiannual'}
+    if (is.null(dateparams$dateGeneration)){dateparams$dateGeneration='Backward'}
+    if (is.null(dateparams$endOfMonth)){dateparams$endOfMonth=0}
+    if (is.null(dateparams$fixingDays)){dateparams$fixingDays=2}
+
     dateparams <- matchParams(dateparams)
-    if (class(discountCurve)=="DiscountCurve"){
-         val <- .Call("QL_FixedRateWithRebuiltCurve",
-                   bond, rates, c(discountCurve$table$date), 
-                  discountCurve$table$zeroRates, dateparams,
-                  PACKAGE="RQuantLib")         
-    }
-    else {
-         if (length(discountCurve)==2){
-             val <- .Call("QL_FixedRateBond1", 
-                     bond, rates, discountCurve, dateparams,
-                     PACKAGE="RQuantLib")
-         }
-         if (length(discountCurve)==3){
-             val <- .Call("QL_FixedRateBond2", 
-                    bond, rates, discountCurve[[1]], 
-                    discountCurve[[2]], discountCurve[[3]],
-                    dateparams,
-                    PACKAGE="RQuantLib")      
-         }
-    }
-    val$cashFlow <- as.data.frame(val$cashFlow)
-    class(val) <- c("FixedRateBond", "Bond")    
+
+    val <- .Call("QL_FixedRateWithRebuiltCurve",
+                 bond, rates, c(discountCurve$table$date),
+                 discountCurve$table$zeroRates, dateparams,
+                 PACKAGE="RQuantLib")
+
+    class(val) <- c("FixedRateBond", "Bond")
     val
 }
 
@@ -149,7 +168,7 @@ FixedRateBondYield <- function( settlementDays, price, faceAmount,
                            compound, redemption, issueDate) {
      UseMethod("FixedRateBondYield")
 }
-FixedRateBondYield.default <- function(settlementDays = 1,price, faceAmount,
+FixedRateBondYield.default <- function(settlementDays = 1,price, faceAmount=100,
                                 effectiveDate, maturityDate,
                                 period, calendar = "us", rates,
                                 dayCounter=2, businessDayConvention=0,
@@ -182,8 +201,8 @@ FixedRateBondPriceByYield <- function( settlementDays, yield, faceAmount,
                            compound, redemption, issueDate) {
      UseMethod("FixedRateBondPriceByYield")
 }
-FixedRateBondPriceByYield.default <- function(settlementDays = 1, yield, faceAmount,
-                                effectiveDate, maturityDate,
+FixedRateBondPriceByYield.default <- function(settlementDays = 1, yield, faceAmount=100,
+                                effectiveDate=issueDate, maturityDate,
                                 period, calendar = "us", rates,
                                 dayCounter=2, businessDayConvention=0,
                                 compound = 0, redemption = 100, issueDate) {
@@ -213,56 +232,59 @@ FloatingRateBond <- function(bond, gearings, spreads, caps, floors,
     UseMethod("FloatingRateBond")
 }
 
-FloatingRateBond.default <- function(bond, gearings, spreads, caps, floors,
-                                     index, curve, dateparams){
+FloatingRateBond.default <- function(bond,
+                                     gearings=c(),
+                                     spreads=c(),
+                                     caps=c(),
+                                     floors=c(),
+                                     index,
+                                     curve,
+                                     dateparams=list(refDate=bond$issueDate-2,
+                                                     settlementDays=1,
+                                                     calendar='us',
+                                                     businessDayConvention='Following',
+                                                     terminationDateConvention='Following',
+                                                     dayCounter='Thirty360',
+                                                     period='Semiannual',
+                                                     dateGeneration='Backward',
+                                                     endOfMonth=0,
+                                                     fixingDays=2)
+                                     ) {
     val <- 0
+
+    if (is.null(bond$faceAmount)){bond$faceAmount=100}
+    if (is.null(bond$redemption)){bond$redemption=100}
+    if (is.null(bond$effectiveDate)){bond$effectiveDate=bond$issueDate}
+
+
+    if (is.null(dateparams$settlementDays)){dateparams$settlementDays=1}
+    if (is.null(dateparams$calendar)){dateparams$calendar='us'}
+    if (is.null(dateparams$businessDayConvention)){
+      dateparams$businessDayConvention='Following'
+    }
+    if (is.null(dateparams$terminationDateConvention)){
+      dateparams$terminationDateConvention='Following'
+    }
+    if (is.null(dateparams$dayCounter)){dateparams$dayCounter='Thirty360'}
+    if (is.null(dateparams$period)){dateparams$period='Semiannual'}
+    if (is.null(dateparams$dateGeneration)){dateparams$dateGeneration='Backward'}
+    if (is.null(dateparams$endOfMonth)){dateparams$endOfMonth=0}
+    if (is.null(dateparams$fixingDays)){dateparams$fixingDays=2}
+    if (is.null(dateparams$refDate)) {dateparams$refDate=bond$issueDate-2}
+
     dateparams <- matchParams(dateparams)
-    if (class(curve)=="DiscountCurve"){
-        indexparams <- list(type=index$type, length=index$length, 
+
+    indexparams <- list(type=index$type, length=index$length,
                         inTermOf=index$inTermOf)
-        ibor <- index$term
-        val <- .Call("QL_FloatingWithRebuiltCurve",
+    ibor <- index$term
+    val <- .Call("QL_FloatingWithRebuiltCurve",
                  bond, gearings, spreads, caps, floors, indexparams,
                  c(ibor$table$date), ibor$table$zeroRates,
-                 c(curve$table$date), curve$table$zeroRates, 
-                 dateparams, 
+                 c(curve$table$date), curve$table$zeroRates,
+                 dateparams,
                  PACKAGE="RQuantLib")
-    }   
-    else {
-         indexparams <- list(type=index$type, length=index$length, 
-                        inTermOf=index$inTermOf)
-         if ((length(curve)==2) && (length(index$term)==2)){
-             val <- .Call("QL_FloatBond1", 
-                     bond, gearings, spreads, caps, floors,
-                     indexparams, index$term, curve, dateparams,
-                     PACKAGE="RQuantLib")
-          }
-          if ((length(curve)==2) && (length(index$term)==3)){
-             ibor <- index$term
-             val <- .Call("QL_FloatBond2", 
-                     bond, gearings, spreads, caps, floors,
-                     indexparams, ibor[[1]], ibor[[2]],
-                     ibor[[3]], curve, dateparams,
-                     PACKAGE="RQuantLib")
-          }
-          if ((length(curve)==3) && (length(index$term)==2)){
-              val <- .Call("QL_FloatBond3", 
-                    bond, gearings, spreads, caps, floors, 
-                    indexparams, index$term, curve[[1]],curve[[2]], 
-                    curve[[3]], dateparams,
-                    PACKAGE="RQuantLib")      
-          }
-          if ((length(curve)==3) && (length(index$term)==3)){
-              ibor <- index$term
-              val <- .Call("QL_FloatBond4", 
-                    bond, gearings, spreads, caps, floors, 
-                    indexparams, ibor[[1]], ibor[[2]], ibor[[3]], 
-                    curve[[1]],curve[[2]], curve[[3]], dateparams,
-                    PACKAGE="RQuantLib")      
-          }
-    }
-    val$cashFlow <- as.data.frame(val$cashFlow)
-    class(val) <- c("FloatingRateBond", "Bond")    
+
+    class(val) <- c("FloatingRateBond", "Bond")
     val
 
 }
@@ -272,24 +294,56 @@ ConvertibleZeroCouponBond <- function(bondparams, process, dateparams){
     UseMethod("ConvertibleZeroCouponBond")
 }
 
-ConvertibleZeroCouponBond.default <- function(bondparams, process, dateparams){
+ConvertibleZeroCouponBond.default <- function(bondparams,
+                                              process,
+                                              dateparams=list(
+                                                settlementDays=1,
+                                                calendar='us',
+                                                dayCounter='Thirty360',
+                                                period='Semiannual',
+                                                businessDayConvention='Following'
+                                                )
+                                              )
+{
     val <- 0
+
+
+    if (is.null(bondparams$exercise)){bondparams$exercise='am'}
+    if (is.null(bondparams$faceAmount)){bondparams$faceAmount=100}
+    if (is.null(bondparams$redemption)){bondparams$redemption=100}
+    if (is.null(bondparams$divSch)){
+      bondparams$divSch = data.frame(Type=character(0), Amount=numeric(0),
+        Rate = numeric(0), Date = as.Date(character(0)))
+    }
+    if (is.null(bondparams$callSch)){
+      bondparams$callSch = data.frame(Price=numeric(0), Type=character(0),
+        Date=as.Date(character(0)))
+    }
+
+
+    if (is.null(dateparams$settlementDays)){dateparams$settlementDays=1}
+    if (is.null(dateparams$calendar)){dateparams$calendar='us'}
+    if (is.null(dateparams$businessDayConvention)){
+      dateparams$businessDayConvention='Following'
+    }
+    if (is.null(dateparams$dayCounter)){dateparams$dayCounter='Thirty360'}
+    if (is.null(dateparams$period)){dateparams$period='Semiannual'}
+
     dateparams <- matchParams(dateparams)
     callabilitySchedule <- bondparams$callSch
     dividendSchedule <- bondparams$divSch
-    dividendYield <- process$divYield    
-    riskFreeRate <- process$rff        
-    val <- .Call("QL_ConvertibleZeroBond", 
+    dividendYield <- process$divYield
+    riskFreeRate <- process$rff
+    val <- .Call("QL_ConvertibleZeroBond",
                     bondparams, process,
-                    c(dividendYield$table$date), 
+                    c(dividendYield$table$date),
                     dividendYield$table$zeroRates,
-                    c(riskFreeRate$table$date), 
+                    c(riskFreeRate$table$date),
                     riskFreeRate$table$zeroRates,
                     dividendSchedule, callabilitySchedule, dateparams,
                     PACKAGE="RQuantLib")
 
-    val$cashFlow <- as.data.frame(val$cashFlow)
-    class(val) <- c("ConvertibleZeroCouponBond", "Bond")    
+    class(val) <- c("ConvertibleZeroCouponBond", "Bond")
     val
 }
 
@@ -298,24 +352,56 @@ ConvertibleFixedCouponBond <- function(bondparams, coupon, process, dateparams){
     UseMethod("ConvertibleFixedCouponBond")
 }
 
-ConvertibleFixedCouponBond.default <- function(bondparams, coupon, process, dateparams){
+ConvertibleFixedCouponBond.default <- function(bondparams,
+                                               coupon,
+                                               process,
+                                               dateparams=list(
+                                                 settlementDays=1,
+                                                 calendar='us',
+                                                 dayCounter='Thirty360',
+                                                 period='Semiannual',
+                                                 businessDayConvention='Following'
+                                                 )
+                                               ){
+
     val <- 0
+
+    if (is.null(bondparams$exercise)){bondparams$exercise='am'}
+    if (is.null(bondparams$faceAmount)){bondparams$faceAmount=100}
+    if (is.null(bondparams$redemption)){bondparams$redemption=100}
+    if (is.null(bondparams$divSch)){
+      bondparams$divSch = data.frame(Type=character(0), Amount=numeric(0),
+        Rate = numeric(0), Date = as.Date(character(0)))
+    }
+    if (is.null(bondparams$callSch)){
+      bondparams$callSch = data.frame(Price=numeric(0), Type=character(0),
+        Date=as.Date(character(0)))
+    }
+
+
+    if (is.null(dateparams$settlementDays)){dateparams$settlementDays=1}
+    if (is.null(dateparams$calendar)){dateparams$calendar='us'}
+    if (is.null(dateparams$businessDayConvention)){
+      dateparams$businessDayConvention='Following'
+    }
+    if (is.null(dateparams$dayCounter)){dateparams$dayCounter='Thirty360'}
+    if (is.null(dateparams$period)){dateparams$period='Semiannual'}
+
     dateparams <- matchParams(dateparams)
     callabilitySchedule <- bondparams$callSch
     dividendSchedule <- bondparams$divSch
-    dividendYield <- process$divYield    
-    riskFreeRate <- process$rff        
-    val <- .Call("QL_ConvertibleFixedBond", 
+    dividendYield <- process$divYield
+    riskFreeRate <- process$rff
+    val <- .Call("QL_ConvertibleFixedBond",
                     bondparams, coupon, process,
-                    c(dividendYield$table$date), 
+                    c(dividendYield$table$date),
                     dividendYield$table$zeroRates,
-                    c(riskFreeRate$table$date), 
+                    c(riskFreeRate$table$date),
                     riskFreeRate$table$zeroRates,
                     dividendSchedule, callabilitySchedule, dateparams,
                     PACKAGE="RQuantLib")
 
-    val$cashFlow <- as.data.frame(val$cashFlow)
-    class(val) <- c("ConvertibleFixedCouponBond", "Bond")    
+    class(val) <- c("ConvertibleFixedCouponBond", "Bond")
     val
 }
 
@@ -323,32 +409,64 @@ ConvertibleFloatingCouponBond <- function(bondparams, iborindex,spread, process,
     UseMethod("ConvertibleFloatingCouponBond")
 }
 
-ConvertibleFloatingCouponBond.default <- function(bondparams, iborindex,spread, process, dateparams){
+ConvertibleFloatingCouponBond.default <- function(bondparams,
+                                                  iborindex,
+                                                  spread,
+                                                  process,
+                                                  dateparams=list(
+                                                    settlementDays=1,
+                                                    calendar='us',
+                                                    dayCounter='Thirty360',
+                                                    period='Semiannual',
+                                                    businessDayConvention='Following'
+                                                    )){
     val <- 0
+
+    if (is.null(bondparams$exercise)){bondparams$exercise='am'}
+    if (is.null(bondparams$faceAmount)){bondparams$faceAmount=100}
+    if (is.null(bondparams$redemption)){bondparams$redemption=100}
+    if (is.null(bondparams$divSch)){
+      bondparams$divSch = data.frame(Type=character(0), Amount=numeric(0),
+        Rate = numeric(0), Date = as.Date(character(0)))
+    }
+    if (is.null(bondparams$callSch)){
+      bondparams$callSch = data.frame(Price=numeric(0), Type=character(0),
+        Date=as.Date(character(0)))
+    }
+
+
+    if (is.null(dateparams$settlementDays)){dateparams$settlementDays=1}
+    if (is.null(dateparams$calendar)){dateparams$calendar='us'}
+    if (is.null(dateparams$businessDayConvention)){
+      dateparams$businessDayConvention='Following'
+    }
+    if (is.null(dateparams$dayCounter)){dateparams$dayCounter='Thirty360'}
+    if (is.null(dateparams$period)){dateparams$period='Semiannual'}
+
+
     dateparams <- matchParams(dateparams)
     callabilitySchedule <- bondparams$callSch
     dividendSchedule <- bondparams$divSch
-    dividendYield <- process$divYield    
-    riskFreeRate <- process$rff        
+    dividendYield <- process$divYield
+    riskFreeRate <- process$rff
 
-    indexparams <- list(type=iborindex$type, length=iborindex$length, 
+    indexparams <- list(type=iborindex$type, length=iborindex$length,
                         inTermOf=iborindex$inTermOf)
     ibor <- iborindex$term
 
-    val <- .Call("QL_ConvertibleFloatingBond", 
+    val <- .Call("QL_ConvertibleFloatingBond",
                     bondparams,  process,
-                    c(dividendYield$table$date), 
+                    c(dividendYield$table$date),
                     dividendYield$table$zeroRates,
-                    c(riskFreeRate$table$date), 
+                    c(riskFreeRate$table$date),
                     riskFreeRate$table$zeroRates,
-                    c(ibor$table$date), 
+                    c(ibor$table$date),
                     ibor$table$zeroRates,
                     indexparams,spread,
                     dividendSchedule, callabilitySchedule, dateparams,
                     PACKAGE="RQuantLib")
 
-    val$cashFlow <- as.data.frame(val$cashFlow)
-    class(val) <- c("ConvertibleFloatingCouponBond", "Bond")    
+    class(val) <- c("ConvertibleFloatingCouponBond", "Bond")
     val
 }
 
@@ -356,34 +474,68 @@ CallableBond <- function(bondparams, hullWhite, coupon, dateparams){
     UseMethod("CallableBond")
 }
 
-CallableBond.default <- function(bondparams, hullWhite, coupon, dateparams){
+CallableBond.default <- function(bondparams, hullWhite,
+                                 coupon,
+                                 dateparams=list(
+                                   settlementDays=1,
+                                   calendar='us',
+                                   dayCounter='Thirty360',
+                                   period='Semiannual',
+                                   businessDayConvention='Following',
+                                   terminationDateConvention='Following'
+                                 )){
     val <- 0
+
+    if (is.null(bondparams$faceAmount)){bondparams$faceAmount=100}
+    if (is.null(bondparams$redemption)){bondparams$redemption=100}
+    if (is.null(bondparams$callSch)){
+      bondparams$callSch = data.frame(Price=numeric(0), Type=character(0),
+        Date=as.Date(character(0)))
+    }
+
+    if (is.null(dateparams$settlementDays)){dateparams$settlementDays=1}
+    if (is.null(dateparams$calendar)){dateparams$calendar='us'}
+    if (is.null(dateparams$businessDayConvention)){
+      dateparams$businessDayConvention='Following'
+    }
+    if (is.null(dateparams$terminationDateConvention)){
+      dateparams$terminationDateConvention='Following'
+    }
+    if (is.null(dateparams$dayCounter)){dateparams$dayCounter='Thirty360'}
+    if (is.null(dateparams$period)){dateparams$period='Semiannual'}
+
     dateparams <- matchParams(dateparams)
-    callSch <- bondparams$callabilitySchedule
+    callSch <- bondparams$callSch
 #    hw.termStructure <- hullWhite$term
-    
+
     val <- .Call("QL_CallableBond", bondparams, hullWhite,coupon,
 #                c(hw.termStructure$table$date),
 #                hw.termStructure$table$zeroRates,
                 callSch, dateparams,
                 PACKAGE="RQuantLib")
-    val$cashFlow <- as.data.frame(val$cashFlow)
-    class(val) <- c("CallableBond", "Bond")    
-    val   
+    class(val) <- c("CallableBond", "Bond")
+    val
 }
 
-FittedBondCurve <- function(curveparams, lengths, coupons, dateparams){
+FittedBondCurve <- function(curveparams,
+                            lengths,
+                            coupons,
+                            marketQuotes,
+                            dateparams){
     UseMethod("FittedBondCurve")
 }
 
-FittedBondCurve.default <- function(curveparams, lengths, coupons, dateparams){
+FittedBondCurve.default <- function(curveparams,
+                                    lengths,
+                                    coupons,
+                                    marketQuotes,
+                                    dateparams){
     val <- 0
     dateparams <- matchParams(dateparams)
     val <- .Call("QL_FittedBondCurve", curveparams,
-                 lengths, coupons, dateparams, PACKAGE="RQuantLib")
+                 lengths, coupons, marketQuotes, dateparams, PACKAGE="RQuantLib")
 
     class(val) <- c("DiscountCurve")
-    val$table <- as.data.frame(val$table)
     val
 }
 
@@ -401,7 +553,7 @@ FittedBondCurve.default <- function(curveparams, lengths, coupons, dateparams){
 #   atmSwapTenors <- swaptionVol$atmSwapTenors
 #   volMatrix <- swaptionVol$volatilityMatrix
 #   swapIndex <- matchParams(swapIndex)
-#   ibor <- iborIndex$term  
+#   ibor <- iborIndex$term
 #   val <- .Call("QL_CMSBond", bondparams, iborIndex, swapIndex, cap, floor, gearings, spreads,
 #                swaptionVol, atmOptionTenors, atmSwapTenors, volMatrix, pricer
 #                ibor$table$dates, ibor$table$zeroRates)
@@ -413,7 +565,7 @@ matchDayCounter <- function(daycounter = c("Actual360", "ActualFixed", "ActualAc
                  "Business252", "OneDayCounter", "SimpleDayCounter", "Thirty360"))
 {
      if (!is.numeric(daycounter)) {
-         daycounter <- match.arg(daycounter)         
+         daycounter <- match.arg(daycounter)
          daycounter <- switch(daycounter,
                               Actual360 = 0,
                               ActualFixed = 1,
@@ -423,7 +575,7 @@ matchDayCounter <- function(daycounter = c("Actual360", "ActualFixed", "ActualAc
                               SimpleDayCounter = 5,
                               Thirty360 = 6)
      }
-     daycounter     
+     daycounter
 }
 
 matchBDC <- function(bdc = c("Following", "ModifiedFollowing",
@@ -431,26 +583,26 @@ matchBDC <- function(bdc = c("Following", "ModifiedFollowing",
                              "Unadjusted")) {
      if (!is.numeric(bdc)){
          bdc <- match.arg(bdc)
-         bdc <- switch(bdc, 
+         bdc <- switch(bdc,
                        Following = 0,
                        ModifiedFollowing = 1,
                        Preceding = 2,
                        ModifiedPreceding = 3,
                        Unadjusted = 4)
      }
-     bdc    
+     bdc
 }
 
-matchCompounding <- function(cp = c("Simple", "Compounded", 
+matchCompounding <- function(cp = c("Simple", "Compounded",
                                     "Continuous", "SimpleThenCompounded")) {
      if (!is.numeric(cp)){
         cp <- match.arg(cp)
         cp <- switch(cp,
-                     Simple = 0, 
+                     Simple = 0,
                      Compounded = 1,
                      Continuous = 2,
                      SimpleThenCompounded = 3)
-     }    
+     }
      cp
 }
 matchFrequency <- function(freq = c("NoFrequency","Once", "Annual",
@@ -460,17 +612,17 @@ matchFrequency <- function(freq = c("NoFrequency","Once", "Annual",
                                     "Weekly", "Daily")){
     if (!is.numeric(freq)){
        freq <- match.arg(freq)
-       freq <- switch(freq, 
-                      NoFrequency = 0, Once = 1, Annual = 2,
-                      Semiannual = 3, EveryFourthMonth = 4,
-                      Quarterly = 5, Bimonthly = 6,
-                      EveryFourthWeek = 7, Biweekly = 8,
-                      Weekly = 9, Daily = 10)
+       freq <- switch(freq,
+                      NoFrequency = -1, Once = 0, Annual = 1,
+                      Semiannual = 2, EveryFourthMonth = 3,
+                      Quarterly = 4, Bimonthly = 6,
+                      Monthly = 12, EveryFourthWeek = 13,
+                      Biweekly = 26, Weekly = 52, Daily = 365)
     }
     freq
 }
 matchDateGen <- function(dg = c("Backward", "Forward", "Zero",
-                                "ThirdWednesday", "Twentieth", 
+                                "ThirdWednesday", "Twentieth",
                                 "TwentiethIMM")){
    if (!is.numeric(dg)){
       dg <- match.arg(dg)
@@ -484,7 +636,7 @@ matchDateGen <- function(dg = c("Backward", "Forward", "Zero",
 
 
 matchParams <- function(params) {
-  
+
   if (!is.null(params$dayCounter)) {
      params$dayCounter <- matchDayCounter(params$dayCounter)
   }
